// NEW: continous transmission if port #2 = "LOW"

// selectable output channels  
// -----------------------------
// Port:   #3      #4      #5
// -----------------------------
// Ch.1:  "HIGH"  "HIGH"  "HIGH"
// Ch.2:  "LOW"   "HIGH"  "HIGH"
// Ch.3:  "HIGH"  "LOW"   "HIGH" 
// Ch,4:  "LOW"   "LOW"   "HIGH" 
// Ch.5:  "HIGH"  "HIGH"  "LOW"
// Ch.6:  "LOW"   "HIGH"  "LOW"
// Ch.7:  "HIGH"  "LOW"   "LOW" 
// Ch,8:  "LOW"   "LOW"   "LOW" 

// http://remco.org/index.php/2017/01/16/adf4351-evaluation-board-microwave-beacon/comment-page-1/#comment-20388
// http://remco.org/ADF435x-attiny13a.ino

// Simple VHF/UHF/SHF beacon with an ADF4351 (evaluation board); Clock= 25MHz 
// Original by (c) PA3FYM, Jan 2017
//  
// Please check the ADF4351 datasheet or consult ADF4351 software
// for proper PLL register values.
//
// This code may be used (by you) for non commercial purposes, 
// and/or intentions, and/or actions as long as you refer to me.

// const byte clk =    12; // PB4 ADF4531 clock  
// const byte data =   11; // PB3 ADF4531 data
// const byte le =     10; // PB2 ADF4531 latch enable

#define cwtick 85          // cw tick speed in msec

long int r0, r1, r2, r3, r4, r5;
 
// gewuenschte Morsezeichensequenz als Folge von Nullen und Einsen zusammenstellen
// Punkt= 1; Strich= 111; Zwischenraum= 0; Beispiel fuer Zeichen "D" ( -.. ) = 1110101
// Pause zwischen Zeichen= 000; Pause zwischen Worten= 00000000 

// DJ7OO   00000000 1110101 000 1011101110111 000 1110111010101 000 11101110111 000 11101110111 00000000
// JN49CX  01011101110111 000 11101 000 10101010111 000 11101110111011101 000 11101011101 000 11101010111 0000     

// jetzt als Bitfolge zusammenfassen (Leerstellen entfernen) und in Bloecke zu je 8 Einzelbits aufteilen

byte message[22]= {0b00000000,0b11101010,0b00101110,0b11101110,0b00111011,0b10101010,0b00111011,0b10111000,
                   0b11101110,0b11100000,0b00001011,0b10111011,0b10001110,0b10001010,0b10101110,0b00111011,
                   0b10111011,0b10100011,0b10101110,0b10001110, 0b10101110,0b00000000};   

// letzten Teilblock ggf. mit Nullen auffuellen; Anzahl der Bloecke zaehlen und Wert zwischen eckigen Klammern eintragen
// gefundenen Wert in Codezeile 182 einsetzen.

////////////////////////////////////////////////////////
void writePLL(uint32_t PLLword) {          // this routine clocks a 32 bits word into the ADF4351
                                           // msb (b31) first, lsb (b0) last 
  for (byte flop=32; flop>0; flop--) {     // PLL word has 32 bits 
    (PLLword & 0x80000000? PORTB |= 0b00001000 : PORTB &= 0b11110111);   // AND with MSB to clock in '1' or '0' on PB3 = 'pin11'                                                                             
    PORTB |= 0b00010000;                   // clock in bit on rising edge of CLK (PB4 = 'pin12' = 1)
    PORTB &= 0b11101111;                   // CLK (PB4 = 0)      
    PLLword <<= 1;                         // rotate left for next bit
} // for flop 
    PORTB |= 0b00000100;                   // latch in PLL word on rising edge of LE (PB2 = 'pin10' = 1)
    PORTB &= 0b11111011;                   // LE (PB2 = 0)      
} // writePLL

////////////////////////////////////////////////////////////////////////////
void setup () {   
Serial.begin(9600);
Serial.println("Start");
pinMode(2, INPUT_PULLUP);
pinMode(3, INPUT_PULLUP);
pinMode(4, INPUT_PULLUP);
pinMode(5, INPUT_PULLUP);
pinMode(13, OUTPUT);

DDRB  = 0xff; // PB7-0 are all outputs 
PORTB = 0x00; // make PB7-0 low

 // select channel 1-8 via #3, #4, #5
 //-------------------------------------------- 
 // Ch.1: Fout= 50.400 MHz 
 if(digitalRead(3)==HIGH&&digitalRead(4)==HIGH&&digitalRead(5)==HIGH)
 { 
   r0 = 0x408018;  
   r1 = 0x80083E9;
   r2 = 0x4E42; 
   r3 = 0x4B3;
   r4 = 0xEC843C;
   r5 = 0x580005; 
 }
 
 // Ch.2: Fout= 70.160 MHz
 else if(digitalRead(3)==LOW&&digitalRead(4)==HIGH&&digitalRead(5)==HIGH)
 { 
   r0 = 0x2C8FB8;  
   r1 = 0x8009389;
   r2 = 0x4E42; 
   r3 = 0x4B3;
   r4 = 0xDC843C;
   r5 = 0x580005; 
 }
 
 // Ch.3: Fout= 144.400 MHz 
 else if(digitalRead(3)==HIGH&&digitalRead(4)==LOW&&digitalRead(5)==HIGH)
  { 
    r0 = 0x2E01A0;  
    r1 = 0x80083E9;
    r2 = 0x4E42; 
    r3 = 0x4B3;
    r4 = 0xCC843C;
    r5 = 0x580005; 
  }

 // Ch.4: Fout= 432.400 MHz 
 else if(digitalRead(3)==LOW&&digitalRead(4)==LOW&&digitalRead(5)==HIGH) 
   { 
     r0 = 0x450170; 
     r1 = 0x80083E9;
     r2 = 0x4E42; 
     r3 = 0x4B3;
     r4 = 0xBC843C;
     r5 = 0x580005; 
   }

// Ch.5: Fout= 1296.815 MHz 
 if(digitalRead(3)==HIGH&&digitalRead(4)==HIGH&&digitalRead(5)==LOW)
 { 
   r0 = 0x33BA38;  
   r1 = 0x800CE21;
   r2 = 0x4E42; 
   r3 = 0x4B3;
   r4 = 0x9C843C;
   r5 = 0x580005; 
 }
 
 // Ch.6: Fout= 2320.8125 MHz
 else if(digitalRead(3)==LOW&&digitalRead(4)==HIGH&&digitalRead(5)==LOW)
 { 
   r0 = 0x2E0A68;  
   r1 = 0x8008C81;
   r2 = 0x4E42; 
   r3 = 0x4B3;
   r4 = 0x8C843C;
   r5 = 0x580005; 
 }
 
 // Ch.7: Fout= 3400.8125 MHz 
 else if(digitalRead(3)==HIGH&&digitalRead(4)==LOW&&digitalRead(5)==LOW)
  { 
    r0 = 0x440068;  
    r1 = 0x8008C81;
    r2 = 0x4E42; 
    r3 = 0x4B3;
    r4 = 0x8C843C;
    r5 = 0x580005; 
  }

 // Ch.8: Fout= 2592.025 MHz x4 = 10368.100MHz
 else if(digitalRead(3)==LOW&&digitalRead(4)==LOW&&digitalRead(5)==LOW) 
   { 
     r0 = 0x339548; 
     r1 = 0x8009F41;
     r2 = 0x4E42; 
     r3 = 0x4B3;
     r4 = 0x8C843C;
     r5 = 0x580005; 
   }
   
     writePLL(r5); 
     writePLL(r4); 
     writePLL(r3); 
     writePLL(r2); 
     writePLL(r1); 
     writePLL(r0);  
   }

/////////////////////////////////////////////////////////////////
void loop() {                              // main loop     
    if(digitalRead(2)==HIGH)
    {
    Serial.println("CW-Start");
    delay(750);                            // pause before transmitting beacon text
    for (int i=0; i<22; i++) {             // message array contains 22 bytes, see definition above
    digitalWrite(13, HIGH);
                                           // if i = uint8_t then code is 8 bytes larger (?!, try it ;-)                             
    byte flop = message[i];                // isolate pending message byte         
    for (uint8_t y=8; y>0; y--) {          // pending message byte has ... yes! ... 8 bits ;-)         
    writePLL(flop & 0x80? r4 : (r4-32));   // key transmitter (ADF4351 Reg4, b5) depending on msb value of flop 
    delay(cwtick);                         // delay cwtick            
    digitalWrite(13, LOW);
          flop <<= 1;                      // rotate left for next (most significant) bit            
     } // for y         
  } // for i     
      delay(350);                          // after beacon text small pause
      writePLL(r4);                        // TX on    
      Serial.println("CW-Stop");
    }
    else
    { writePLL(r4);}                        // TX on    
      delay(15000);                         // for 15 seconds
    }





